@@ -7,3 +7,7 @@ Exception classes for the tx client.
 
 class UnInitializedError(Exception):
     """The project directory has not been initialized."""
+
+
+class UnknownCommandError(Exception):
+    """The provided command is not supported."""
