@@ -106,6 +106,9 @@ def pull_parser():
         dest="minimum_perc", default=0,
         help="Specify the minimum acceptable percentage of a translation "
              "in order to download it.")
+    parser.add_option("--pseudo", action="store_true", dest="pseudo",
+        default=False,
+        help="Apply this option to download a pseudo file.")
     parser.add_option(
         "--mode", action="store", dest="mode", help=(
             "Specify the mode of the translation file to pull (e.g. "
@@ -190,7 +193,7 @@ def set_parser():
             "Specify the i18n type of the resource(s). This is only needed, if "
             "the resource(s) does not exist yet in Transifex. For a list of "
             "available i18n types, see "
-            "http://help.transifex.net/features/formats.html"
+            "http://help.transifex.com/features/formats.html"
         )
     )
     parser.add_option("--minimum-perc", action="store", dest="minimum_perc",
@@ -202,7 +205,7 @@ def set_parser():
     parser.add_option(
         "--mode", action="store", dest="mode", help=(
             "Specify the mode of the translation file to pull (e.g. "
-            "'reviewed'). See http://help.transifex.net/features/client/"
+            "'reviewed'). See http://help.transifex.com/features/client/"
             "index.html#defining-the-mode-of-the-translated-file for the"
             "available values."
         )
