@@ -1,18 +1,20 @@
 import os, sys, re, errno
+import ssl
 try:
     from json import loads as parse_json, dumps as compile_json
 except ImportError:
     from simplejson import loads as parse_json, dumps as compile_json
-import urllib2 # This should go and instead use do_url_request everywhere
 
-from urls import API_URLS
-from txclib.log import logger
+from txclib.packages import urllib3
+from txclib.urls import API_URLS
 from txclib.exceptions import UnknownCommandError
+from txclib.paths import posix_path, native_path, posix_sep
+from txclib.web import user_agent_identifier
+from txclib.log import logger
 
 
-def find_dot_tx(path = os.path.curdir, previous = None):
-    """
-    Return the path where .tx folder is found.
+def find_dot_tx(path=os.path.curdir, previous=None):
+    """Return the path where .tx folder is found.
 
     The 'path' should be a DIRECTORY.
     This process is functioning recursively from the current directory to each
@@ -32,13 +34,13 @@ def find_dot_tx(path = os.path.curdir, previous = None):
 # Parse file filter expressions and create regex
 
 def regex_from_filefilter(file_filter, root_path = os.path.curdir):
-    """
-    Create proper regex from <lang> expression
-    """
+    """Create proper regex from <lang> expression."""
     # Force expr to be a valid regex expr (escaped) but keep <lang> intact
-    expr_re = re.escape(os.path.join(root_path, file_filter))
+    expr_re = re.escape(
+        posix_path(os.path.join(root_path, native_path(file_filter)))
+    )
     expr_re = expr_re.replace("\\<lang\\>", '<lang>').replace(
-        '<lang>', '([^%(sep)s]+)' % { 'sep': re.escape(os.path.sep)})
+        '<lang>', '([^%(sep)s]+)' % { 'sep': re.escape(posix_sep)})
 
     return "^%s$" % expr_re
 
@@ -55,13 +57,14 @@ def parse_tx_url(url):
     Try to match given url to any of the valid url patterns specified in
     TX_URLS. If not match is found, we raise exception
     """
-    for type in TX_URLS.keys():
-        pattern = TX_URLS[type]
+    for type_ in TX_URLS.keys():
+        pattern = TX_URLS[type_]
         m = re.match(pattern, url)
         if m:
-            return type, m.groupdict()
-
-    raise Exception("tx: Malformed url given. Please refer to our docs: http://bit.ly/txautor")
+            return type_, m.groupdict()
+    raise Exception(
+        "tx: Malformed url given. Please refer to our docs: http://bit.ly/txautor"
+    )
 
 
 def get_details(api_call, username, password, *args, **kwargs):
@@ -70,30 +73,24 @@ def get_details(api_call, username, password, *args, **kwargs):
 
     This function can also be used to check the existence of a project.
     """
-    import base64
     url = (API_URLS[api_call] % (kwargs)).encode('UTF-8')
-
-    req = urllib2.Request(url=url)
-    base64string = base64.encodestring('%s:%s' % (username, password))[:-1]
-    authheader = "Basic %s" % base64string
-    req.add_header("Authorization", authheader)
-
+    conn = urllib3.connection_from_url(kwargs['hostname'])
+    headers = urllib3.util.make_headers(
+        basic_auth='{0}:{1}'.format(username, password),
+        accept_encoding=True,
+        user_agent=user_agent_identifier(),
+    )
     try:
-        fh = urllib2.urlopen(req)
-        raw = fh.read()
-        fh.close()
-        remote_project = parse_json(raw)
-    except urllib2.HTTPError, e:
-        if e.code in [401, 403, 404]:
-            raise e
-        else:
-            # For other requests, we should print the message as well
-            raise Exception("Remote server replied: %s" % e.read())
-    except urllib2.URLError, e:
-        error = e.args[0]
-        raise Exception("Remote server replied: %s" % error[1])
-
-    return remote_project
+        r = conn.request('GET', url, headers=headers)
+        remote_project = parse_json(r.data)
+        r.close()
+        return remote_project
+    except ssl.SSLError:
+        logger.error("Invalid SSL certificate")
+        raise
+    except Exception, e:
+        logger.debug(unicode(e))
+        raise
 
 
 def valid_slug(slug):
@@ -204,56 +201,12 @@ def color_text(text, color_name, bold=False):
         return text
 
 
-##############################################
-# relpath implementation taken from Python 2.7
+def files_in_project(curpath):
+    """
+    Iterate over the files in the project.
 
-if not hasattr(os.path, 'relpath'):
-    if os.path is sys.modules.get('ntpath'):
-        def relpath(path, start=os.path.curdir):
-            """Return a relative version of a path"""
-
-            if not path:
-                raise ValueError("no path specified")
-            start_list = os.path.abspath(start).split(os.path.sep)
-            path_list = os.path.abspath(path).split(os.path.sep)
-            if start_list[0].lower() != path_list[0].lower():
-                unc_path, rest = os.path.splitunc(path)
-                unc_start, rest = os.path.splitunc(start)
-                if bool(unc_path) ^ bool(unc_start):
-                    raise ValueError("Cannot mix UNC and non-UNC paths (%s and %s)"
-                                                                        % (path, start))
-                else:
-                    raise ValueError("path is on drive %s, start on drive %s"
-                                                        % (path_list[0], start_list[0]))
-            # Work out how much of the filepath is shared by start and path.
-            for i in range(min(len(start_list), len(path_list))):
-                if start_list[i].lower() != path_list[i].lower():
-                    break
-            else:
-                i += 1
-
-            rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
-            if not rel_list:
-                return os.path.curdir
-            return os.path.join(*rel_list)
-
-    else:
-        # default to posixpath definition
-        def relpath(path, start=os.path.curdir):
-            """Return a relative version of a path"""
-
-            if not path:
-                raise ValueError("no path specified")
-
-            start_list = os.path.abspath(start).split(os.path.sep)
-            path_list = os.path.abspath(path).split(os.path.sep)
-
-            # Work out how much of the filepath is shared by start and path.
-            i = len(os.path.commonprefix([start_list, path_list]))
-
-            rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
-            if not rel_list:
-                return os.path.curdir
-            return os.path.join(*rel_list)
-else:
-    from os.path import relpath
+    Return each file under ``curpath`` with its absolute name.
+    """
+    for root, dirs, files in os.walk(curpath, followlinks=True):
+        for f in files:
+            yield os.path.abspath(os.path.join(root, f))
